@@ -370,7 +370,8 @@ match.default <- function(x, ...) base::match(x, ...)
 #' @rdname match
 #' @export
 match.rvar <- function(x, ...) {
-  draws_of(x) <- while_preserving_dims(base::match, draws_of(x), ...)
+  .draws <- draws_of(x)
+  draws_of(x) <- copy_dims(.draws, base::match(.draws, ...))
   x
 }
 
@@ -381,6 +382,59 @@ match.rvar <- function(x, ...) {
 }
 
 
+# rvar_ifelse -------------------------------------------------------------
+
+#' Random variable ifelse
+#'
+#' A version of `ifelse()` that returns an [`rvar`].
+#'
+#' @param test (logical [`rvar`], or castable to one) logical test determining
+#' whether the value in `yes` or `no` is assigned in the corresponding position
+#' of the result.
+#' @param yes ([`rvar`], or castable to one) corresponding values assigned for
+#' entries in `test` that are `TRUE`.
+#' @param no ([`rvar`], or castable to one) corresponding values assigned for
+#' entries in `test` that are `FALSE`.
+#'
+#' @returns
+#' An [`rvar`] with the common type of `yes` and `no` (as determined by
+#' `vctrs::vec_cast_common()`) and a shape determined by broadcasting `test`,
+#' `yes`, and `no` to a common shape (see the section on broadcasting rules in
+#' `vignette("rvar")`). For every element of `draws_of(test)`, the corresponding
+#' element of `draws_of(yes)` or `draws_of(no)` is placed into the result,
+#' depending on whether the element of `test` is `TRUE` or `FALSE`.
+#'
+#' @examples
+#' x <- rvar(1:4)
+#' y <- rvar(5:8)
+#'
+#' i <- rvar(c(TRUE,FALSE,TRUE,FALSE))
+#' z <- rvar_ifelse(i, x, y)
+#' z
+#' draws_of(z)
+#' @importFrom vctrs vec_cast_common
+#' @export
+rvar_ifelse = function(test, yes, no) {
+  test <- as_rvar(test)
+  yes <- as_rvar(yes)
+  no <- as_rvar(no)
+  if (!is.logical(draws_of(test))) {
+    stop_no_call("`rvar_ifelse(test, yes, no)` requires `test` to be a logical rvar, or castable to one.")
+  }
+  c(yes, no) %<-% vec_cast_common(yes, no)
+  c(test, yes, no) %<-% conform_array_dims(conform_rvar_ndraws(list(test, yes, no)))
+
+  test_draws <- draws_of(test)
+  false_draws <- test_draws %in% FALSE
+  draws_of(yes)[false_draws] <- draws_of(no)[false_draws]
+
+  na_draws <- is.na(test_draws)
+  draws_of(yes)[na_draws] <- NA_real_
+
+  yes
+}
+
+
 # ggplot2::scale_type -----------------------------------------------------
 
 # This generic is not exported here as {ggplot2} is only in Suggests, so
@@ -434,7 +488,7 @@ check_rvar_yank_index = function(x, i, ...) {
     stop_no_call("Missing indices not allowed with `[[` in an rvar.")
   } else if (any(vapply(index, is.logical, logical(1)))) {
     stop_no_call("Logical indices not allowed with `[[` in an rvar.")
-  } else if (any(vapply(index, function(x) x < 0, logical(1)))) {
+  } else if (any(vapply(index, function(x) is.numeric(x) && x < 0, logical(1)))) {
     stop_no_call("subscript out of bounds")
   }
 
@@ -509,12 +563,10 @@ conform_rvar_nchains <- function(rvars) {
   rvars
 }
 
-# given two rvars, conform their number of draws and chains
+# given two rvars, conform their number of draws
 # so they can be used together (or throw an error if they can't be)
 # @param keep_constants keep constants as 1-draw rvars
-conform_rvar_ndraws_nchains <- function(rvars, keep_constants = FALSE) {
-  rvars <- conform_rvar_nchains(rvars)
-
+conform_rvar_ndraws <- function(rvars, keep_constants = FALSE) {
   # broadcast to a common number of chains. If keep_constants = TRUE,
   # constants will not be broadcast.
   .ndraws = Reduce(ndraws2_common, lapply(rvars, ndraws))
@@ -525,6 +577,15 @@ conform_rvar_ndraws_nchains <- function(rvars, keep_constants = FALSE) {
   rvars
 }
 
+# given multiple rvars, conform their number of draws and chains
+# so they can be used together (or throw an error if they can't be)
+# @param keep_constants keep constants as 1-draw rvars
+conform_rvar_ndraws_nchains <- function(rvars, keep_constants = FALSE) {
+  rvars <- conform_rvar_nchains(rvars)
+  rvars <- conform_rvar_ndraws(rvars)
+  rvars
+}
+
 # Check that the first rvar can be conformed to the dimensions of the second,
 # ignoring 1s
 check_rvar_dims_first <- function(x, y) {
@@ -594,6 +655,13 @@ dim_common <- function(dims) {
   Reduce(dim2_common, dims)
 }
 
+# given a list of arrays, broadcast their dimensions to be equal
+conform_array_dims <- function(arrays) {
+  .dim <- dim_common(lapply(arrays, dim))
+  lapply(arrays, broadcast_array, .dim)
+}
+
+#' @importFrom rlang inject missing_arg
 broadcast_array  <- function(x, dim, broadcast_scalars = TRUE) {
   if (!broadcast_scalars && length(x) == 1) {
     # quick exit: not broadcasting scalars; return them as vectors
@@ -601,10 +669,8 @@ broadcast_array  <- function(x, dim, broadcast_scalars = TRUE) {
     return(x)
   }
 
-  current_dim <- dim(x)
+  current_dim <- dim(x) %||% length(x)
   current_dimnames <- dimnames(x)
-  current_levels <- levels(x)
-  current_class <- oldClass(x)
 
   if (length(current_dim) < length(dim)) {
     # add dimensions of size 1 as necessary so we can broadcast those
@@ -623,7 +689,7 @@ broadcast_array  <- function(x, dim, broadcast_scalars = TRUE) {
     )
   }
 
-  dim_to_broadcast = which(current_dim != dim)
+  dim_to_broadcast <- which(current_dim != dim)
 
   if (length(dim_to_broadcast) == 0) {
     # quick exit: already has desired dim or just needed extra dims on the end
@@ -638,26 +704,26 @@ broadcast_array  <- function(x, dim, broadcast_scalars = TRUE) {
     )
   }
 
-  # move the dims we aren't broadcasting to the front so they are recycled properly
-  perm <- c(seq_along(dim)[-dim_to_broadcast], dim_to_broadcast)
-
-  # broadcast the other dims
-  x <- array(aperm(x, perm), dim[perm])
-
-  # move dims back to their original order
-  x <- aperm(x, order(perm))
-
   if (!is.null(current_dimnames)) {
-    # restore any dimnames that we did not have to broadcast
-    dim_to_restore <- current_dim == dim
-    dimnames(x)[dim_to_restore] <- current_dimnames[dim_to_restore]
+    # the slice (below) should correctly preserve dimnames that are not broadcast,
+    # but it will also duplicate dimnames that were broadcast --- which is probably
+    # undesirable as those dimnames are no longer unique. So, drop those before
+    # we do the broadcast (this can be much faster than dropping afterwards if
+    # the broadcast dimensions are large)
+    dimnames(x)[dim_to_broadcast] <- list(NULL)
   }
 
-  # restore class and levels
-  levels(x) <- current_levels
-  oldClass(x) <- current_class
+  # construct the indices used in a slice that will broadcast the array
+  # e.g. if we want to broadcast x with dim c(2,1,2,4) to dim c(2,3,2,4), we
+  # need to do the slice x[,c(1,1,1),,], which broadcasts the second dimension
+  # to length 3
+  # first, construct the list of missing arguments
+  indices <- rep(list(missing_arg()), length(dim))
+  # second, fill in the appropriate number of 1s in the dims we want to broadcast
+  indices[dim_to_broadcast] <- lapply(dim[dim_to_broadcast], rep, x = 1L)
 
-  x
+  # do the slice to broadcast the array
+  inject(x[!!!indices, drop = FALSE])
 }
 
 # broadcast the draws dimension of an rvar to the requested size
@@ -677,55 +743,6 @@ broadcast_draws <- function(x, .ndraws, keep_constants = FALSE) {
   }
 }
 
-# flatten dimensions and names of an array
-flatten_array = function(x, x_name = NULL) {
-  # determine new dimension names in the form x,y,z
-  # start with numeric names
-  dimname_lists <- lapply(dim(x), seq_len)
-  .dimnames <- dimnames(x)
-  if (!is.null(.dimnames)) {
-    # where character names are provided, use those instead of the numeric names
-    dimname_lists = lapply(seq_along(dimname_lists), function(i) .dimnames[[i]] %||% dimname_lists[[i]])
-  }
-  # expand out the dimname lists into the appropriate combinations and assemble into new names
-  dimname_grid <- expand.grid(dimname_lists)
-  new_names <- apply(dimname_grid, 1, paste0, collapse = ",")
-
-  .length <- length(x)
-  old_dim <- dim(x)
-  dim(x) <- .length
-
-  # update variable names
-  if (is.null(x_name)) {
-    # no base name for x provided, just use index names
-    names(x) <- new_names
-  } else if (.length == 1 && (isTRUE(old_dim == 1) || length(old_dim) == 0)) {
-    # scalar, use the provided base name
-    names(x) <- x_name
-  } else if (.length >= 1) {
-    # rename the variables with their indices in brackets
-    names(x) <- paste0(x_name, "[", new_names %||% seq_along(x), "]")
-  }
-
-  x
-}
-
-#' Fast conversion of rvar draws to a flattened data frame. Equivalent to
-#' as.data.frame(draws_of(flatten_array(x, name))) except it works with
-#' factor rvars (as.data.frame does not work on array-like factors)
-#' @noRd
-flatten_rvar_draws_to_df <- function(x, x_name = NULL) {
-  if (length(x) == 0) {
-    out <- data.frame(row.names = draw_ids(x))
-  } else {
-    draws <- draws_of(flatten_array(x, x_name))
-    cols <- lapply(seq_len(ncol(draws)), function(i) unname(draws[, i]))
-    names(cols) <- colnames(draws)
-    out <- vctrs::new_data_frame(cols)
-  }
-  out
-}
-
 #' copy the dimension names (and name of the dimension) from dimension src_i
 #' in array src to dimension dst_i in array dst
 #' @noRd
@@ -783,6 +800,13 @@ drop_chain_dim <- function(x) {
 #' (first one is draws), etc.
 #' @noRd
 cleanup_rvar_draws <- function(x) {
+  assert(
+    check_numeric(x),
+    check_logical(x),
+    check_factor(x),
+    check_character(x)
+  )
+
   if (length(x) == 0) {
     # canonical NULL rvar is at least 1 draw of nothing
     # this ensures that (e.g.) extending a null rvar
@@ -807,35 +831,35 @@ cleanup_rvar_draws <- function(x) {
 
   # if x is a character array, make it a factor
   if (is.character(x)) {
-    x <- while_preserving_dims(factor, x)
+    x <- copy_dims(x, factor(x))
   }
 
   x
 }
 
-#' Execute x <- f(x, ...) but preserve dimensions and dimension names of x.
-#' Useful for functions that do not change the length of x but which drop
-#' dimensions.
+#' Copy dims and dimnames of `src` to `dst`.
+#' Useful for functions that do not change the length of a variable but which
+#' drop dimensions.
+#' @param src a variable, possibly with dims and dimnames
+#' @param dst a variable of same length as `src`.
 #' @noRd
-while_preserving_dims <- function(f, x, ...) {
-  .dim <- dim(x)
-  .dimnames <- dimnames(x)
-  x <- f(x, ...)
-  dim(x) <- .dim
-  dimnames(x) <- .dimnames
-  x
+copy_dims <- function(src, dst) {
+  dim(dst) <- dim(src)
+  dimnames(dst) <- dimnames(src)
+  dst
 }
 
-#' Execute x <- f(x, ...) but preserve class and levels of x.
-#' Useful for functions that do not change the length of x but which levels.
+#' Copy class and levels of src to dst. Class is copied to ensure that the
+#' status of the variable as a factor or ordered is maintained.
+#' Useful for functions that do not change the shape of a variable but which
+#' drop levels.
+#' @param src a variable, possibly with levels
+#' @param dst a variable of same shape as `src`.
 #' @noRd
-while_preserving_levels <- function(f, x, ...) {
-  .class <- oldClass(x)
-  .levels <- levels(x)
-  x <- f(x, ...)
-  oldClass(x) <- .class
-  levels(x) <- .levels
-  x
+copy_levels <- function(src, dst) {
+  oldClass(dst) <- oldClass(src)
+  levels(dst) <- levels(src)
+  dst
 }
 
 #' a version of apply() that works on factor-like arrays
@@ -899,7 +923,7 @@ summarise_rvar_within_draws_via_matrix <- function(x, .name, .f, ..., .ordered_o
   if (.ordered_okay && is_rvar_ordered(x)) {
     .levels <- levels(x)
     .draws <- .f(draws_of(as_rvar_numeric(x)), ...)
-    .draws <- while_preserving_dims(function(.draws) ordered(.levels[round(.draws)], .levels), .draws)
+    .draws <- copy_dims(.draws, ordered(.levels[round(.draws)], .levels))
   } else if (is_rvar_factor(x)) {
     stop_no_call("Cannot apply `", .name, "` function to rvar_factor objects.")
   } else {
